<%!
    from flask import g, request
    from copyvios.attribution import get_attribution_info
    from copyvios.checker import T_POSSIBLE, T_SUSPECT
    from copyvios.misc import cache
%>\
<%
    titleparts = []
    if query.page:
        titleparts.append(query.page.title)
    titleparts.append("Earwig's Copyvio Detector")
    title = " | ".join(titleparts)
%>\
<%include file="/support/header.mako" args="title=title, splash=not result"/>
<%namespace module="copyvios.highlighter" import="highlight_delta"/>\
<%namespace module="copyvios.misc" import="httpsfix, urlstrip"/>\
% if notice:
    <div id="notice-box" class="gray-box">
        ${notice}
    </div>
% endif
% if query.submitted:
    % if query.error:
        <div id="info-box" class="red-box"><p>
            % if query.error == "bad action":
                Unknown action: <b><span class="mono">${query.action | h}</span></b>.
            % elif query.error == "no search method":
                No copyvio search methods were selected. A check can only be made using the search engine, links present in the page, Turnitin, or some combination of these.
            % elif query.error == "bad oldid":
                The revision ID <code>${query.oldid | h}</code> is invalid. It should be an integer.
            % elif query.error == "no URL":
                Compare mode requires a URL to be entered. Enter one in the text box below, or choose copyvio search mode to look for content similar to the article elsewhere on the web.
            % elif query.error == "bad URI":
                Unsupported URI scheme: <a href="${query.url | h}">${query.url | h}</a>.
            % elif query.error == "no data":
                Couldn't find any text in <a href="${query.url | h}">${query.url | h}</a>. <i>Note:</i> only HTML documents, plain text pages, and PDFs are supported, and content generated by JavaScript or found inside iframes is ignored.
            % elif query.error == "timeout":
                The URL <a href="${query.url | h}">${query.url | h}</a> timed out before any data could be retrieved.
            % elif query.error == "search error":
                An error occurred while using the search engine (${query.exception}). <i>Note:</i> there is a daily limit on the number of search queries the tool is allowed to make. You may <a href="${request.url | httpsfix, h}&amp;use_engine=0">repeat the check without using the search engine</a>.
            % else:
                An unknown error occurred.
            % endif
        </p></div>
    % elif not query.site:
        <div id="info-box" class="red-box">
            <p>The given site (project=<b><span class="mono">${query.project | h}</span></b>, language=<b><span class="mono">${query.lang | h}</span></b>) doesn't seem to exist. It may also be closed or private. <a href="https://${query.lang | h}.${query.project | h}.org/">Confirm its URL.</a></p>
        </div>
    % elif query.oldid and not result:
        <div id="info-box" class="red-box">
            <p>The revision ID couldn't be found: <a href="https://${query.site.domain | h}/w/index.php?oldid=${query.oldid | h}">${query.oldid | h}</a>.</p>
        </div>
    % elif query.title and not result:
        <div id="info-box" class="red-box">
            <p>The page couldn't be found: <a href="${query.page.url}">${query.page.title | h}</a>.</p>
        </div>
    % endif
%endif
<p>This tool attempts to detect <a href="https://en.wikipedia.org/wiki/WP:COPYVIO">copyright violations</a> in articles. In <i>search mode</i>, it will check for similar content elsewhere on the web using <a href="https://developers.google.com/custom-search/">Google</a>, external links present in the text of the page, or <a href="https://en.wikipedia.org/wiki/Wikipedia:Turnitin">Turnitin</a> (via <a href="https://en.wikipedia.org/wiki/User:EranBot">EranBot</a>), depending on which options are selected. In <i>compare mode</i>, the tool will compare the article to a specific webpage without making additional searches, like the <a href="https://dupdet.toolforge.org/">Duplication Detector</a>.</p>
<p>Running a full check can take up to a minute if other websites are slow or if the tool is under heavy use. Please be patient. If you get a timeout, wait a moment and refresh the page.</p>
<p>Be aware that other websites can copy from Wikipedia, so check the results carefully, especially for older or well-developed articles. Specific websites can be skipped by adding them to the <a href="https://en.wikipedia.org/wiki/User:EarwigBot/Copyvios/Exclusions">excluded URL list</a>.</p>
<form id="cv-form" action="${request.script_root}/" method="get">
    <div class="oo-ui-layout oo-ui-horizontalLayout">
        <label class="site oo-ui-widget oo-ui-widget-enabled oo-ui-labelElement-label oo-ui-labelElement oo-ui-labelWidget">Site</label>
        <div class="oo-ui-widget oo-ui-widget-enabled oo-ui-inputWidget oo-ui-dropdownInputWidget oo-ui-dropdownInputWidget-php">
            <select name="lang" required="" class="oo-ui-inputWidget-input oo-ui-indicator-down" title="Language">
                <% selected_lang = query.orig_lang if query.orig_lang else g.cookies["CopyviosDefaultLang"].value if "CopyviosDefaultLang" in g.cookies else cache.bot.wiki.get_site().lang %>\
                % for code, name in cache.langs:
                    % if code == selected_lang:
                        <option value="${code | h}" selected="selected">${name}</option>
                    % else:
                        <option value="${code | h}">${name}</option>
                    % endif
                % endfor
            </select>
        </div>
        <div class="oo-ui-widget oo-ui-widget-enabled oo-ui-inputWidget oo-ui-dropdownInputWidget oo-ui-dropdownInputWidget-php">
            <select name="project" required="" class="oo-ui-inputWidget-input oo-ui-indicator-down" title="Project">
                <% selected_project = query.project if query.project else g.cookies["CopyviosDefaultProject"].value if "CopyviosDefaultProject" in g.cookies else cache.bot.wiki.get_site().project %>\
                % for code, name in cache.projects:
                    % if code == selected_project:
                        <option value="${code | h}" selected="selected">${name}</option>
                    % else:
                        <option value="${code | h}">${name}</option>
                    % endif
                % endfor
            </select>
        </div>
    </div>
    <div class="oo-ui-layout oo-ui-horizontalLayout">
        <label for="cv-title" class="page oo-ui-widget oo-ui-widget-enabled oo-ui-labelElement-label oo-ui-labelElement oo-ui-labelWidget">Page</label>
        <div class="page-title oo-ui-widget oo-ui-widget-enabled oo-ui-inputWidget oo-ui-textInputWidget oo-ui-textInputWidget-type-text oo-ui-textInputWidget-php">
            <input id="cv-title" type="text" class="oo-ui-inputWidget-input" name="title" placeholder="Title" title="Page title"
                % if query.title:
                    value="${query.page.title if query.page else query.title | h}"
                % endif
            >
        </div>
        <label class="oo-ui-widget oo-ui-widget-enabled oo-ui-labelElement-label oo-ui-labelElement oo-ui-labelWidget">or</label>
        <div class="page-oldid oo-ui-widget oo-ui-widget-enabled oo-ui-inputWidget oo-ui-textInputWidget oo-ui-textInputWidget-type-text oo-ui-textInputWidget-php">
            <input id="cv-oldid" type="text" class="oo-ui-inputWidget-input" name="oldid" placeholder="Revision ID" title="Revision ID"
                % if query.oldid:
                    value="${query.oldid | h}"
                % endif
            >
        </div>
    </div>
    <div class="oo-ui-layout oo-ui-horizontalLayout">
        <span class="oo-ui-widget oo-ui-widget-enabled">
            <span class="oo-ui-widget oo-ui-widget-enabled oo-ui-inputWidget oo-ui-radioInputWidget">
                <input id="action-search" class="oo-ui-inputWidget-input" type="radio" name="action" value="search" ${'checked="checked"' if (query.action == "search" or not query.action) else ""}><span></span>
            </span>
            <label for="action-search" class="action oo-ui-widget oo-ui-widget-enabled oo-ui-labelElement-label oo-ui-labelElement oo-ui-labelWidget">Copyvio search</label>
        </span>

        <input type="hidden" name="use_engine" value="0">
        <span class="oo-ui-widget oo-ui-widget-enabled">
            <span class="cv-search-oo-ui oo-ui-widget oo-ui-widget-enabled oo-ui-inputWidget oo-ui-checkboxInputWidget">
                <input id="cv-cb-engine" class="cv-search oo-ui-inputWidget-input" type="checkbox" name="use_engine" value="1" ${'checked="checked"' if query.use_engine not in ("0", "false") else ""}>
                <span class="oo-ui-checkboxInputWidget-checkIcon oo-ui-widget oo-ui-widget-enabled oo-ui-iconElement-icon oo-ui-icon-check oo-ui-iconElement oo-ui-labelElement-invisible oo-ui-iconWidget oo-ui-image-invert"></span>
            </span>
            <label for="cv-cb-engine" class="oo-ui-widget oo-ui-widget-enabled oo-ui-labelElement-label oo-ui-labelElement oo-ui-labelWidget">Use search engine</label>
        </span>

        <input type="hidden" name="use_links" value="0">
        <span class="oo-ui-widget oo-ui-widget-enabled">
            <span class="cv-search-oo-ui oo-ui-widget oo-ui-widget-enabled oo-ui-inputWidget oo-ui-checkboxInputWidget">
                <input id="cv-cb-links" class="cv-search oo-ui-inputWidget-input" type="checkbox" name="use_links" value="1" ${'checked="checked"' if query.use_links not in ("0", "false") else ""}>
                <span class="oo-ui-checkboxInputWidget-checkIcon oo-ui-widget oo-ui-widget-enabled oo-ui-iconElement-icon oo-ui-icon-check oo-ui-iconElement oo-ui-labelElement-invisible oo-ui-iconWidget oo-ui-image-invert"></span>
            </span>
            <label for="cv-cb-links" class="oo-ui-widget oo-ui-widget-enabled oo-ui-labelElement-label oo-ui-labelElement oo-ui-labelWidget">Use links in page</label>
        </span>

        <input type="hidden" name="turnitin" value="0">
        <span class="oo-ui-widget oo-ui-widget-enabled">
            <span class="cv-search-oo-ui oo-ui-widget oo-ui-widget-enabled oo-ui-inputWidget oo-ui-checkboxInputWidget">
                <input id="cv-cb-turnitin" class="cv-search oo-ui-inputWidget-input" type="checkbox" name="turnitin" value="1" ${'checked="checked"' if query.turnitin in ("1", "true") else ""}>
                <span class="oo-ui-checkboxInputWidget-checkIcon oo-ui-widget oo-ui-widget-enabled oo-ui-iconElement-icon oo-ui-icon-check oo-ui-iconElement oo-ui-labelElement-invisible oo-ui-iconWidget oo-ui-image-invert"></span>
            </span>
            <label for="cv-cb-turnitin" class="oo-ui-widget oo-ui-widget-enabled oo-ui-labelElement-label oo-ui-labelElement oo-ui-labelWidget">Use Turnitin</label>
        </span>
    </div>
    <div class="oo-ui-layout oo-ui-horizontalLayout">
        <span class="oo-ui-widget oo-ui-widget-enabled">
            <span class="oo-ui-widget oo-ui-widget-enabled oo-ui-inputWidget oo-ui-radioInputWidget">
                <input id="action-compare" class="oo-ui-inputWidget-input" type="radio" name="action" value="compare" ${'checked="checked"' if query.action == "compare" else ""}><span></span>
            </span>
            <label for="action-compare" class="action oo-ui-widget oo-ui-widget-enabled oo-ui-labelElement-label oo-ui-labelElement oo-ui-labelWidget">Copyvio compare</label>
        </span>
        <div class="compare-url cv-compare-oo-ui oo-ui-widget oo-ui-widget-enabled oo-ui-inputWidget oo-ui-textInputWidget oo-ui-textInputWidget-type-text oo-ui-textInputWidget-php">
            <input type="text" class="cv-compare oo-ui-inputWidget-input" name="url" placeholder="URL" title="URL to compare"
                % if query.url:
                    value="${query.url | h}"
                % endif
            >
        </div>
    </div>
    % if query.nocache or (result and result.cached):
        <div class="oo-ui-layout oo-ui-horizontalLayout">
            <span class="cv-search-oo-ui oo-ui-widget oo-ui-widget-enabled oo-ui-inputWidget oo-ui-checkboxInputWidget">
                <input id="cb-nocache" class="oo-ui-inputWidget-input" type="checkbox" name="nocache" value="1" ${'checked="checked"' if query.nocache else ""}>
                <span class="oo-ui-checkboxInputWidget-checkIcon oo-ui-widget oo-ui-widget-enabled oo-ui-iconElement-icon oo-ui-icon-check oo-ui-iconElement oo-ui-labelElement-invisible oo-ui-iconWidget oo-ui-image-invert"></span>
            </span>
            <label for="cb-nocache">Bypass cache</label>
        </div>
    % endif
    <div class="oo-ui-layout oo-ui-horizontalLayout">
        <span class="oo-ui-widget oo-ui-widget-enabled oo-ui-buttonElement oo-ui-buttonElement-framed oo-ui-labelElement oo-ui-flaggedElement-primary oo-ui-flaggedElement-progressive oo-ui-buttonWidget">
            <button type="submit" class="oo-ui-inputWidget-input oo-ui-buttonElement-button">
                <span class="oo-ui-labelElement-label">Submit</span>
            </button>
        </span>
    </div>
</form>

% if result:
    <div id="generation-time">
        Results
        % if result.cached:
            <a id="cv-cached" href="#">cached<span>To save time (and money), this tool will retain the results of checks for up to 72 hours. This includes the URLs of the checked sources, but neither their content nor the content of the article. Future checks on the same page (assuming it remains unchanged) will not involve additional search queries, but a fresh comparison against the source URL will be made. If the page is modified, a new check will be run.</span></a> from <abbr title="${result.cache_time}">${result.cache_age} ago</abbr>. Originally
        % endif
        generated in <span class="mono">${round(result.time, 3)}</span>
        % if query.action == "search":
            seconds using <span class="mono">${result.queries}</span> quer${"y" if result.queries == 1 else "ies"}.
        % else:
            seconds.
        % endif
        <a href="${request.script_root | h}/?lang=${query.lang | h}&amp;project=${query.project | h}&amp;oldid=${query.oldid or query.page.lastrevid | h}&amp;action=${query.action | h}&amp;${"use_engine={0}&use_links={1}".format(int(query.use_engine not in ("0", "false")), int(query.use_links not in ("0", "false"))) if query.action == "search" else "" | h}${"url=" if query.action == "compare" else ""}${query.url if query.action == "compare" else "" | u}">Permalink.</a>
    </div>

    <div id="cv-result" class="${'red' if result.confidence >= T_SUSPECT else 'yellow' if result.confidence >= T_POSSIBLE else 'green'}-box">
        <table id="cv-result-head-table">
            <colgroup>
                <col>
                <col>
                <col>
            </colgroup>
            <tr>
                <td>
                    <a href="${query.page.url}">${query.page.title | h}</a>
                    % if query.oldid:
                        @<a href="https://${query.site.domain | h}/w/index.php?oldid=${query.oldid | h}">${query.oldid | h}</a>
                    % endif
                    % if query.redirected_from:
                        <br />
                        <span id="redirected-from">Redirected from <a href="https://${query.site.domain | h}/w/index.php?title=${query.redirected_from.title | u}&amp;redirect=no">${query.redirected_from.title | h}</a>. <a href="${request.url | httpsfix, h}&amp;noredirect=1">Check original.</a></span>
                    % endif
                </td>
                <td>
                    <div>
                        % if result.confidence >= T_SUSPECT:
                            Violation suspected
                        % elif result.confidence >= T_POSSIBLE:
                            Violation possible
                        % elif result.sources:
                            Violation unlikely
                        % else:
                            No violation
                        % endif
                    </div>
                    <div>${round(result.confidence * 100, 1)}%</div>
                    <div>similarity</div>
                </td>
                <td>
                    % if result.url:
                        <a href="${result.url | h}">${result.url | urlstrip, h}</a>
                    % else:
                        <span id="result-head-no-sources">No matches found.</span>
                    % endif
                </td>
            </tr>
        </table>
    </div>

    <% attrib = get_attribution_info(query.site, query.page) %>
    % if attrib:
        <div id="attribution-warning" class="yellow-box">
            This article contains an attribution template: <code>{{<a href="${attrib[1]}">${attrib[0] | h}</a>}}</code>. Please verify that any potential copyvios are not from properly attributed sources.
        </div>
    % endif

    % if query.turnitin_result:
        <div id="turnitin-container" class="${'red' if query.turnitin_result.reports else 'green'}-box">
            <div id="turnitin-title">Turnitin Results</div>
            % if query.turnitin_result.reports:
                <table id="turnitin-table"><tbody>
                % for report in turnitin_result.reports:
                    <tr><td class="turnitin-table-cell"><a href="https://eranbot.toolforge.org/ithenticate.py?rid=${report.reportid}">Report ${report.reportid}</a> for text added at <a href="https://${query.lang}.wikipedia.org/w/index.php?title=${query.title}&amp;diff=${report.diffid}"> ${report.time_posted.strftime("%H:%M, %d %B %Y (UTC)")}</a>:
                    <ul>
                    % for source in report.sources:
                          <li>${source['percent']}% of revision text (${source['words']} words) found at <a href="${source['url'] | h}">${source['url'] | h}</a></li>
                    % endfor
                    </ul></td></tr>
                % endfor
                </tbody></table>
            % else:
                <div id="turnitin-summary">No matching sources found.</div>
            % endif
        </div>
    % endif

    % if query.action == "search":
        <% skips = False %>
        <div id="sources-container">
            <div id="sources-title">Checked Sources</div>
            % if result.sources:
                <table id="cv-result-sources">
                    <colgroup>
                        <col>
                        <col>
                        <col>
                    </colgroup>
                    <tr>
                        <th>URL</th>
                        <th>Similarity</th>
                        <th>Compare</th>
                    </tr>
                    % for i, source in enumerate(result.sources):
                        <tr ${'class="source-default-hidden"' if i >= 10 else 'id="source-row-selected"' if i == 0 else ""}>
                            <td><a ${'id="source-selected"' if i == 0 else ""} class="source-url" href="${source.url | h}">${source.url | h}</a></td>
                            <td>
                                % if source.excluded:
                                    <span class="source-excluded">Excluded</span>
                                % elif source.skipped:
                                    <% skips = True %>
                                    <span class="source-skipped">Skipped</span>
                                % else:
                                    <span class="source-similarity ${"source-suspect" if source.confidence >= T_SUSPECT else "source-possible" if source.confidence >= T_POSSIBLE else "source-novio"}">${round(source.confidence * 100, 1)}%</span>
                                % endif
                            </td>
                            <td>
                                <a href="${request.script_root | h}/?lang=${query.lang | h}&amp;project=${query.project | h}&amp;oldid=${query.oldid or query.page.lastrevid | h}&amp;action=compare&amp;url=${source.url | u}">Compare</a>
                            </td>
                        </tr>
                    % endfor
                </table>
            % else:
                <div class="cv-source-footer">
                    No sources checked.
                </div>
            % endif
            % if len(result.sources) > 10:
                <div id="cv-additional" class="cv-source-footer">
                    ${len(result.sources) - 10} URL${"s" if len(result.sources) > 11 else ""} with lower similarity hidden. <a id="show-additional-sources" href="#">Show them.</a>
                </div>
            % endif
            % if skips or result.possible_miss:
                <div class="cv-source-footer">
                    The search ended early because a match was found with high similarity. <a href="${request.url | httpsfix, h}&amp;noskip=1">Do a complete check.</a>
                </div>
            % endif
        </div>
    % endif
    <table id="cv-chain-table">
        <tr>
            <td class="cv-chain-cell">Article: <div class="cv-chain-detail"><p>${highlight_delta(result.article_chain, result.best.chains[1] if result.best else None)}</p></div></td>
            <td class="cv-chain-cell">Source: <div class="cv-chain-detail"><p>${highlight_delta(result.best.chains[0], result.best.chains[1]) if result.best else ""}</p></div></td>
        </tr>
    </table>
% endif
<%include file="/support/footer.mako"/>
